//gcc -lnsl -lresolv -lcurses audio.c -o audio && ./audio 127.0.0.1 1024 Skyrim.mp3

#include "../TCP_Client.c"

#include "math.h"
#include "libc.h"
#include "minimp3.h"
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>

#define SCREEN_W 25
#define SCREEN_H 11
#define BITSHIFT 1 //increase this to reduce stutter
#define UPPER_BOUND 70
#define LOWER_BOUND (25)
#define MAX_FREQ 10000

//Verzögerung der Bildschirmausgabe zum matchen des Tons
#define FIFO_DEPTH 20

#define PI_CONST 3.14159265 

unsigned char** pixel_buf;
struct fourier_data {
	int n;
	float* twiddle_cos; float* twiddle_sin;
	float* fft_out;
	int* bins;
	//float* sin_factor_E; float* cos_factor_E;
	//float* sin_factor_O; float* cos_factor_O;
	} fft_data;

size_t strlen(const char *s);
#define out(text) write(1, (const void *) text, strlen(text))

/*void to_display()
{
	int i,j;
	static unsigned char FBuf[FIFO_DEPTH][SCREEN_W][SCREEN_H];
	static unsigned FHead = 0;
	static unsigned FTail = 0;

	//neuen Frame in Fifo schieben
	FHead = (FHead+1)%FIFO_DEPTH;
	for (i=0;i<SCREEN_H;i++)
	{
		for(j=0;j<SCREEN_W;j++)
			FBuf[FHead][j][i] = pixel_buf[i][j]*255;
	}
	
	//Frame aus FIFO holen
	FTail = (FHead+1)%FIFO_DEPTH;
	resetScreen();
	for (i=0;i<SCREEN_H;i++)
	{
		for(j=0;j<SCREEN_W;j++)
			//putPixel(j,i,pixel_buf[i][j]*255);
			putPixel(j,i,FBuf[FTail][j][i]);
	}
	display();
}*/

void init_fft(int length, int sample_rate)
{
	printf("\nfft init with %i points\n",length);
	fft_data.n = length;
	fft_data.twiddle_cos = malloc(length*sizeof(float));
	fft_data.twiddle_sin = malloc(length*sizeof(float));
	fft_data.fft_out = malloc((length/4)*sizeof(float));
	fft_data.bins = malloc((SCREEN_W+1) * sizeof(int));
	
	//constants for odd datapoints
	int k;
	for (k=0;k<length;k++)
	{
		fft_data.twiddle_cos[k] = sin(-2*PI_CONST/length*k);
		fft_data.twiddle_sin[k] = cos(-2*PI_CONST/length*k);
	}
	
	//compute freqency borders in fft
	int bin_width = sample_rate/(length/2);
	//printf("Freqrange of one bin: %i\n",bin_width);
	int d_bin = MAX_FREQ/bin_width/SCREEN_W;
	//printf("Number of bins in one column: %i\n",d_bin);
	
	for (k=0;k<SCREEN_W+1;k++)
		fft_data.bins[k] = k*d_bin;
}

void fft(short* data_in, int data_length, mp3_info_t info)
{
	int k,m;
	
	if (data_length != fft_data.n)
		init_fft(data_length,info.sample_rate);
	
	for (k=0; k<data_length/4; k++)
	{
		float even_real = 0.0;
		float even_imag = 0.0;
		float odd_real = 0.0;
		float odd_imag = 0.0;
		
		for (m=0; m<data_length/2-1;m+=4)
		{
			float c_real = cos(-PI_CONST*m*k/data_length);
			float c_imag = sin(-PI_CONST*m*k/data_length);
			even_real += data_in[m] * c_real;
			even_imag += data_in[m] * c_imag;
			odd_real += data_in[m+2] * c_real;
			odd_imag += data_in[m+2] * c_imag;		
		}
		
		odd_real *= fft_data.twiddle_cos[k];
		odd_imag *= fft_data.twiddle_sin[k];
		
		float real = (even_real + odd_real)/(data_length/2);
		float imag = (even_imag + odd_imag)/(data_length/2);
		//fft_data.fft_out[k] = sqrt((real*real) + (imag*imag));
		fft_data.fft_out[k] = 10*log10((real*real) + (imag*imag)); 
		//printf("%i: %f+%fi = %f\n",k,real,imag,(fft_data.fft_out[k]));
		//sleep(1);
	}
	//printf("\n");
	
}

int last2N(int n)
{
	int m = n;
	while(n)
	{
		m = n;
		n = m & (m-1);
	}
	return m;
}

void visualize(short* audio_buf, mp3_info_t info)
{
	/*//printf("Length: %i\n",audio_bytes);
	int n = last2N(audio_bytes)>>1;
	//printf("2N: 	%i\n\n",n);
	//int n = audio_bytes-1;
	int i,j,m=0;
	//printf("Get Data\n");
	for (i=0;i<n;i=i+2)
	{
		m += buf[i];
	}
	m = m/(n);
	//printf("Mean: %i\n",m);
	m = m>1000?1000:m;
	m = m<-1000?-1000:m;
	int h = m*(SCREEN_H-1)/(2000)+(SCREEN_H-1)/2;
	for (i=0;i<SCREEN_H;i++)
		pixel_buf[i][SCREEN_W-1] = 0;
	pixel_buf[h][SCREEN_W-1] = 1;
	printf("Mean: %i, pixel set at %i\n",m,h);
	for (i=0;i<SCREEN_H;i++)
	{
		for (j=0;j<SCREEN_W-1;j++)
			pixel_buf[i][j] = pixel_buf[i][j+1];
	}
	to_display();*/
	
	int i,j,n = last2N(info.audio_bytes)>>BITSHIFT;
	//int bin_length = (n/4)/SCREEN_W;
	fft(audio_buf,n,info);
	
	static unsigned char FBuf[FIFO_DEPTH][SCREEN_W][SCREEN_H];
	static unsigned FHead = 0;
	static unsigned FTail = 0;
	
	//neuen Frame in Fifo schieben
	FHead = (FHead+1)%FIFO_DEPTH;
	
	resetScreen();
	for(i=0;i<SCREEN_W;i++)
	{
		float m = 0.0;
		for (j=fft_data.bins[i];j<fft_data.bins[i+1];j++)
		{
			m += fft_data.fft_out[j];
		}
		m /= fft_data.bins[i+1]-fft_data.bins[i];
		
		//printf("%i: %f\n",i,m);
		//sleep(1);
		m = m>UPPER_BOUND?UPPER_BOUND:m;
		m = m<LOWER_BOUND?LOWER_BOUND:m;
		int h = (m-LOWER_BOUND)*(-SCREEN_H-1)/(UPPER_BOUND-LOWER_BOUND)+(SCREEN_H-1);
		for(j=0;j<SCREEN_H;j++)
		{
			if (j>h)
				//putPixel(i,j,255);
				FBuf[FHead][i][j] = 255;
			else
				//putPixel(i,j,0);	
				FBuf[FHead][i][j] = 0;	
		}
	}
	
	//Frame aus FIFO holen
	FTail = (FHead+1)%FIFO_DEPTH;
	resetScreen();
	for (i=0;i<SCREEN_H;i++)
	{
		for(j=0;j<SCREEN_W;j++)
			//putPixel(j,i,pixel_buf[i][j]*255);
			putPixel(j,i,FBuf[FTail][j][i]);
	}
	display();
	
	/*
	short m = 10*log10(fft_data.fft_out[0]);
	//printf("Mean: %i\n",m);
	m = m>1000?1000:m;
	m = m<-1000?-1000:m;
	int h = m*(SCREEN_H-1)/(2000)+(SCREEN_H-1)/2;
	for (i=0;i<SCREEN_H;i++)
		pixel_buf[i][SCREEN_W-1] = 0;
	pixel_buf[h][SCREEN_W-1] = 1;
	printf("Mean: %i, pixel set at %i\n",m,h);
	for (i=0;i<SCREEN_H;i++)
	{
		for (j=0;j<SCREEN_W-1;j++)
			pixel_buf[i][j] = pixel_buf[i][j+1];
	}
	to_display();*/
}

int main(int argc, char *argv[])
{
	if(argc!=4)
	{
		printf("Usage: ./audio <Servername> <Serverport> <Song.mp3>\n");
		exit(1);
	}
	init(argv[1],atoi(argv[2]));
	
	int i;
	pixel_buf = malloc(SCREEN_H * sizeof(unsigned char*));
	for (i=0;i<SCREEN_H;i++)
		pixel_buf[i] = calloc(SCREEN_W,sizeof(unsigned char));
	
	mp3_decoder_t mp3;
	mp3_info_t info;
	int fd, pcm;
	void *file_data;
	unsigned char *stream_pos;
	signed short sample_buf[MP3_MAX_SAMPLES_PER_FRAME];
	int bytes_left;
	int frame_size;
	int value;
	
	//out("minimp3 -- a small MPEG-1 Audio Layer III player based on ffmpeg\n\n");

	fd = open(argv[3], O_RDONLY);
	if (fd < 0) {
		printf("Error: cannot open %s!\n",argv[3]);
		return 1;
	}

	bytes_left = lseek(fd, 0, SEEK_END);    
	file_data = mmap(0, bytes_left, PROT_READ, MAP_PRIVATE, fd, 0);
	stream_pos = (unsigned char *) file_data;
	bytes_left -= 100;
	printf("Now Playing: %s",argv[3]);

	mp3 = mp3_create();
	frame_size = mp3_decode(mp3, stream_pos, bytes_left, sample_buf, &info);
	if (!frame_size) {
		printf("\nError: not a valid MP3 audio file!\n");
		return 1;
	}

	#define FAIL(msg) { \
	printf("\nError: " msg "\n"); \
	return 1; \
	}   

	pcm = open("/dev/dsp", O_WRONLY);
	if (pcm < 0) 
		FAIL("cannot open DSP");

	value = AFMT_S16_LE;
	if (ioctl(pcm, SNDCTL_DSP_SETFMT, &value) < 0)
		FAIL("cannot set audio format");

	if (ioctl(pcm, SNDCTL_DSP_CHANNELS, &info.channels) < 0)
		FAIL("cannot set audio channels");

	if (ioctl(pcm, SNDCTL_DSP_SPEED, &info.sample_rate) < 0)
		FAIL("cannot set audio sample rate");
	
	int n = last2N(frame_size)>>BITSHIFT; 
	init_fft(n,info.sample_rate);
	
	//out("\n\nPress Ctrl+C to stop playback.\n");

	while ((bytes_left >= 0) && (frame_size > 0)) {
		stream_pos += frame_size;
		bytes_left -= frame_size;
		write(pcm, (const void *) sample_buf, info.audio_bytes);
		frame_size = mp3_decode(mp3, stream_pos, bytes_left, sample_buf, NULL);
		visualize(sample_buf,info);
		
	}
	
	printf("\nStopped playing\n");
	
	close(pcm);
	return 0;
}
