#include "fingermodbus_client.h"

//timeout in usec zum UDP Empfangen
#define TIMEOUT 100000

//Debugausgaben oder nicht
#define MODBG 0

static char sendarray[12] = {0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x00, 0x04, 0x01, 0x08, 0x00, 0x02};
static int sockfd;
static struct sockaddr_in serv_addr; // connector's address information
static struct hostent *server;

static float get_payload(char *array);
static unsigned getdiff(struct timeval *first, struct timeval *second);

int modbusfinger_init(char *adress){

    //IP Adresse beim DNS anfragen
    if ((server=gethostbyname(adress)) == NULL) {
        printf("gethostbyname failed");
        return MOD_DNS_FAILED;
    }

   	//Socket erstellen  
    sockfd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
    if (sockfd != -1){
    	#if MODBG
    	printf("socket created\n");
    	#endif
    }else{
    	printf("socket failed\n");
    	return MOD_SOCKET_FAILED;
    }

    //setup transport address
    serv_addr.sin_family = AF_INET;     
    serv_addr.sin_port = htons(502);
    serv_addr.sin_addr = *((struct in_addr *)server->h_addr_list[0]);
    memset(serv_addr.sin_zero, '\0', sizeof serv_addr.sin_zero);

	return MOD_OK;    
}   
   
int modbusfinger_getdata(unsigned short adress, float *ergebnis){

    size_t serv_addr_size = sizeof(struct sockaddr_in);
    char empfangsarray[15];
    int success;
    unsigned usecs = 0;
    struct timeval anfangstime, nowtime;
    
    //Adresse einbetten
    adress *= 2;
    sendarray[8] = (adress>>8);
    sendarray[9] = adress;
    
   	//Request senden
	success = sendto(sockfd, &sendarray, sizeof(sendarray), 0, 
		(const struct sockaddr *)&serv_addr, serv_addr_size);
   	if (success > 0){
   		#if MODBG
   		printf("socket written\n");
   		#endif
   	}else{
   		printf("socket not written\n");
   		return MOD_SOCKETWRITE_FAILED;
   	}
   	
   	//Antwort empfangen (timeout)
	memset(empfangsarray,'\0', sizeof(empfangsarray));
	gettimeofday(&anfangstime,(struct timezone *)0);
	success = -1;
	while((success < 0) && (usecs < TIMEOUT)){
		success = recvfrom(sockfd, empfangsarray, sizeof(empfangsarray), MSG_DONTWAIT, 
			(struct sockaddr *)&serv_addr, &serv_addr_size);
		//Timeout berechnen
		if (success < 0){
			gettimeofday(&nowtime,(struct timezone *)0);
			usecs = getdiff(&anfangstime, &nowtime);
			//printf("usecs: %u\n", usecs);
		}
	}
	if (usecs >= TIMEOUT){
		return MOD_SOCKETREAD_TIMEOUT;
	}
	
	#if MODBG
	printf("empfangen: %u\n", success);	
	printf("empfangsarray: ");
	for (unsigned i = 0; i < 15; i++){
		printf("0x%2x ", empfangsarray[i]&0xff);
	}
	printf("\n");
	printf("ergebnis: %f\n", get_payload(empfangsarray));
	#endif
	
	*ergebnis = get_payload(empfangsarray);
	
	return MOD_OK;
}
	
int modbusfinger_close(void){
    close(sockfd);
    #if MODBG
    printf("socket closed\n");
    #endif

    return MOD_OK;
}

static float get_payload(char *array){
	#define EOFFS 9
	float floating;
	char *floatchars = (char*)&floating;
	floatchars[0] = array[EOFFS+1];
	floatchars[1] = array[EOFFS+0];
	floatchars[2] = array[EOFFS+3];
	floatchars[3] = array[EOFFS+2];
	return floating;
}

static unsigned getdiff(struct timeval *first, struct timeval *second){

	unsigned erg;

	if (first->tv_sec < second->tv_sec){
		//sekunden sind bereits fortgeschritten -> usecs über Sekundenabschnitt berechnen
		erg  = (first->tv_sec - second->tv_sec)*(unsigned)1000000;
		erg += ((unsigned)1000000 - first->tv_usec);
		erg += second->tv_usec;
	}else{
		erg = second->tv_usec - first->tv_usec;
	}
	
	return erg;	

}
