#ifndef _FINGERMODBUS_CLIENT_H_
#define _FINGERMODBUS_CLIENT_H_

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h>

/*
* Öffnet den UDP Socket zu der IP oder dem Hostnamen adress
* adress: 	String der IP oder Hostname enthält
* return: 	MOD_OK bei geöffnetem Socket
*			MOD_DNS_FAILED bei fehlgeschlagener DNS Auflösung
*			MOD_SOCKET_FAILED bei fehlgeschlagenem Socketbind
*/
int modbusfinger_init(char *adress);

/*
* Holt die Daten vom Stromzähler und blockiert solange keine Antwort kommt
* adress:	Eine Datenfeldadresse von den unten aufgeführten Defines
* ergebnis:	Ein Zeiger auf einen Float, enthält nach der Ausführung den Wert aus dem Stromzähler bei returntem MOD_OK
* return:	MOD_OK bei gültigem Wert in ergebnis
*			MOD_SOCKETWRITE_FAILED wenn kein Paket gesendet werden konnte 
*			MOD_SOCKETREAD_FAILED wenn kein Paket empfangen werden konnte
*/
int modbusfinger_getdata(unsigned short adress, float *ergebnis);

/*
* Schließt den Socket
* return: MOD_OK, der Socket ist geschlossen
*/
int modbusfinger_close(void);

//Defines der Verfügbaren Daten des Stromzählers
//Ströme der einzelnen Phasen (A)
#define I_L1 128
#define I_L2 129
#define I_L3 130
//Spannung Phase zu Nulleiter (V)
#define U_L1N 131
#define U_L2N 132
#define U_L3N 133
//Spannung Phase zu Phase (V)
#define U_L1_L2 152
#define U_L2_L3 153
#define U_L3_L1 154
//Wirkleistung der einzelnen Phasen (W)
#define P_L1 134
#define P_L2 135
#define P_L3 136
//Gesamtwirkleistung (W)
#define P_GES 137
//Blindleistung der einzelnen Phasen (Var)
#define Q_L1 138
#define Q_L2 139
#define Q_L3 140
//Gesamtblindleistung (VAr)
#define Q_GES 141
//Scheinleistung der einzelnen Phasen (VA)
#define S_L1 142
#define S_L2 143
#define S_L3 144
//Gesamtscheinleistung (VA)
#define S_GES 145
//Frequenz der einzelnen Phasen (Hz)
#define F_L1 146
#define F_L2 147
#define F_L3 148
//Phasenwinkel der einzelnen Phasen (Grad)
#define PA_UI_L1 149
#define PA_UI_L2 150
#define PA_UI_L3 151
//Maximalspannung Phase zu Nulleiter (V)
#define U_L1N_MAX 155
#define U_L2N_MAX 156
#define U_L3N_MAX 157
//Minimalspannung Phase zu Nulleiter (V)
#define U_L1N_MIN 158
#define U_L2N_MIN 159
#define U_L3N_MIN 160
//Maximalwirkleistung der einzelnen Phasen (W)
#define P_L1_MAX 161
#define P_L2_MAX 162
#define P_L3_MAX 163
//Gesamtarbeit (Wh)
#define ARBEIT 164


//Errordefines
#define MOD_OK 					0
#define MOD_DNS_FAILED 			1
#define MOD_SOCKET_FAILED 		2
#define MOD_SOCKETWRITE_FAILED 	3
#define MOD_SOCKETREAD_FAILED 	4
#define MOD_SOCKETREAD_TIMEOUT 	5

#endif //_FINGERMODBUS_CLIENT_H_ 
