#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <time.h>

#include "fingermodbus_client.h"
#include "TCP_Client.h"

struct dispwert {  
	char uschrift[50];
	unsigned short adress;
	//Floatingpointanzeigen Formatangaben (müssen auf 6 Zeichen passen!)
	unsigned after; //Ziffern nach dem Komma
	float divider; //zB um kWh statt Wh anzuzeigen
};

//Array aus structs was angezeigt werden soll
struct dispwert anzeigeelemente[] = {
	{"Stromverbrauch (kwh)", ARBEIT, 2, 1000.0},
	{"U L3N", U_L3N, 2, 1.0},
	{"P GES kw", P_GES, 3, 1000.0}
};

//Länge des dynamischen Array herausfinden
const unsigned elements_anzeigeelemente = sizeof(anzeigeelemente)/sizeof(struct dispwert);

int showdata(struct dispwert *data);

int main(int argc, char *argv[]) {

	if(argc != 3)
	{
		printf("Usage: ./test <Servername> <Port>\n");
		exit(1);
	}
	
	init((argv[1]), atoi(argv[2]));

	if (modbusfinger_init("dkeipp.rz9.de")){
		printf("Modbus init failed!\n");
		exit(1);
	}
	
	printf("Modbus running, showing up to %u Measurements\n", elements_anzeigeelemente);
	
	for (unsigned i = 0; i < elements_anzeigeelemente; i++){
		
		//Daten anzeigen wenn kein UDP Timeout
		showdata(&anzeigeelemente[i]);
		
	}
	
	modbusfinger_close();
}

int showdata(struct dispwert *data){

	char anzeigewert[7];
	int spalten_uschrift;
	float wert = 0.0;
	
	//Laufschrift für Überschrift und Wert fest anzeigen
	spalten_uschrift = 4*strlen(data->uschrift) - 1;
	//printf ("uschrift: %s, slen: %u\n", data->uschrift, spalten_uschrift);
	
	//Laufschrift doch ned nötig?
	if(spalten_uschrift - 7*4 < 0){
		
		//statischer Text
		for (int i = 0; i < 10; i++){
			if (modbusfinger_getdata(data->adress, &wert)){
				wert = 0.0;
			}
			snprintf(anzeigewert, sizeof(anzeigewert), "%06.*f", data->after, wert/data->divider);
			resetScreen();
			scprintf(data->uschrift, 0, 0, 255);
			scprintf(anzeigewert, 0, 6, 255);
			display();
			sleep(1);
		}
		
	}else{
		
		//Laufschrift durchscrollen
		for (int j = 0; j < 100; j++){
		
			for (int i = 0; i < spalten_uschrift + 7; i++){
				
				//neuen Wert holen alle 1s
				if(0 == ((i*j)%10)){
					if (modbusfinger_getdata(data->adress, &wert)){
						wert = 0.0;
					}
					snprintf(anzeigewert, sizeof(anzeigewert), "%06.*f", data->after, wert/data->divider);
				}
				//nur 10s lang anzeigen und dann rausspringen
				if((0 == ((i*j)%100)) && (j != 0) && (i != 0)){
					return 0;
				}
				
				resetScreen();
				scprintf(data->uschrift, -1*i, 0, 255);
				scprintf(data->uschrift, (-1*i) + spalten_uschrift + 7, 0, 255);
				scprintf(anzeigewert, 0, 6, 255);
				display();
				//printf("i: %i, %i \n", i, spalten_uschrift - 7*4);
				usleep(100000);
			}
		}
	}
	
	usleep(100000);
	
	return 0;
	
}
